use actix_multipart::form::{text::Text, MultipartForm};
use actix_web::{post, HttpResponse};
use lettre::{
    message::MultiPart,
    transport::smtp::authentication::{Credentials, Mechanism},
    Message, SmtpTransport, Transport,
};

#[derive(Debug, MultipartForm, Default)]
struct ParentForm {
    #[multipart(rename = "firstname")]
    first_name: Option<Text<String>>,
    #[multipart(rename = "lastname")]
    last_name: Option<Text<String>>,
    #[multipart(rename = "studentfirstname")]
    student_first_name: Option<Text<String>>,
    #[multipart(rename = "studentlastname")]
    student_last_name: Option<Text<String>>,
    email: Option<Text<String>>,
    authority: Option<Text<String>>,
    positive: Option<Text<String>>,
    negative: Option<Text<String>>,
    #[multipart(rename = "family-relation")]
    family_relation: Option<Text<String>>,
    #[multipart(rename = "previous-trip-info")]
    previous_trip_info: Option<Text<String>>,
    #[multipart(rename = "trip-feelings")]
    trip_feelings: Option<Text<String>>,
    #[multipart(rename = "extra-info")]
    notes: Option<Text<String>>,
}

#[post("/parent-form")]
pub async fn parent_form(MultipartForm(form): MultipartForm<ParentForm>) -> HttpResponse {
    let first = form.first_name.as_ref().unwrap().0.clone();
    let last = form.last_name.as_ref().unwrap().0.clone();
    let student = format!(
        "{} {}",
        form.student_first_name.as_ref().unwrap().0.clone(),
        form.student_last_name.as_ref().unwrap().0.clone()
    );
    let email_subject = format!(
        "{} {} filled out a parent form for {}!",
        first, last, student
    );
    let email = form
        .email
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let authority = form
        .authority
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let positive = form
        .positive
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let negative = form
        .negative
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let family = form
        .family_relation
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let previous_trip_info = form
        .previous_trip_info
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let feelings = form
        .trip_feelings
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    let notes = form
        .notes
        .as_ref()
        .unwrap_or(&Text(String::from("")))
        .0
        .clone();
    log::info!("{first} {last} filled out a parent form!");
    let email = markup::new! {
        @markup::doctype()
            html {
                head {
                    title { @format!("{} {} filled out a Parent Reference form for {}!", first, last, student) }
                    style {
                        "table { border-collapse: collapse; width: 100% }"
                            "td, th { padding: 8px }"
                            "td { text-align: left; width: 70%; word-wrap: break-word }"
                            "th { text-align: right; border-right: 1px solid #ddd }"
                            "tr { border-bottom: 1px solid #ddd }"
                            "h1 { text-align: center }"
                    }
                }
                body {
                    h1 { @format!("Parent Reference form for {}!", student) }
                    hr;
                    table {
                        tr {
                            th { "Name" }
                            td { @format!("{} {}", first, last) }
                        }
                        tr {
                            th { "Student" }
                            td { @student }
                        }
                        tr {
                            th { "Email" }
                            td { @email }
                        }
                        tr {
                            th { "Response to authority" }
                            td { @authority }
                        }
                        tr {
                            th { "Positive Attributes" }
                            td { @positive }
                        }
                        tr {
                            th { "Negaitive Attributes" }
                            td { @negative }
                        }
                        tr {
                            th { "Relations to Family" }
                            td { @family }
                        }
                        tr {
                            th { "Previous trip info" }
                            td { @previous_trip_info }
                        }
                        tr {
                            th { "Feelings about the trip" }
                            td { @feelings }
                        }
                        tr {
                            th { "Other Notes" }
                            td { @notes }
                        }
                    }
                }
            }
    };
    let multi = MultiPart::alternative_plain_html(String::from("Testing"), email.to_string());

    if let Ok(m) = Message::builder()
        .from(
            "TFC ADMIN <no-reply@mail.tfcconnection.org>"
                .parse()
                .unwrap(),
        )
        .to("Chris Cochrun <chris@tfcconnection.org>".parse().unwrap())
        .to("Ethan Rose <ethan@tfcconnection.org>".parse().unwrap())
        .subject(email_subject)
        .multipart(multi)
    {
        let sender = SmtpTransport::relay("mail.tfcconnection.org")
            .ok()
            .unwrap()
            .credentials(Credentials::new(
                "no-reply@mail.tfcconnection.org".to_owned(),
                "r9f36mNZFtiW4f".to_owned(),
            ))
            .authentication(vec![Mechanism::Plain])
            .build();
        match sender.send(&m) {
            Ok(res) => log::info!("{:?}", res),
            Err(e) => log::info!("{e}"),
        }
    } else {
        log::info!("Email incorrect");
    }

    HttpResponse::Ok().body("hi")
}
